import os
import sys
import zipfile

scanDirs = ["$LOGS$", "$LDMAP$"] if (len(sys.argv) > 1 and sys.argv[1].find("database") != -1) else ["$LOGS$"]

def friendlySize(num) :
        for x in ['bytes','KB','MB','GB','TB'] :
                if num < 1024.0:
                        return "{0:3.1f} {1}".format(num, x)
                num /= 1024.0

def zipFiles(srcFilePaths, dstZipPath, maxZipSize) :
        print("Archive \"{0}\" :".format(dstZipPath))
        if os.path.exists(dstZipPath) : os.remove(dstZipPath) # overwrite
        zip = zipfile.ZipFile(dstZipPath, "w", zipfile.ZIP_DEFLATED)
        for srcRootFolderPath, srcRelFilePath in srcFilePaths :
                srcAbsFilePath = os.path.expanduser(os.path.join(srcRootFolderPath, srcRelFilePath))
                print("\tAdding: {0}, \"{1}\" as \"{2}\"...".format(friendlySize(os.path.getsize(srcAbsFilePath)), srcAbsFilePath, srcRelFilePath))
                zip.write(srcAbsFilePath, srcRelFilePath, zipfile.ZIP_DEFLATED)
                zipFileLength = os.path.getsize(dstZipPath)
                print("\t\tZip now {0}".format(friendlySize(zipFileLength)))
                if (zipFileLength > maxZipSize) :
                        print("Zip exceeded max length of - not adding any more files".format(friendlySize(maxZipSize)))
                        break;

def includeFilename(filename) :
        return os.path.splitext(filename)[1] == ".crash" or os.path.splitext(filename)[1] == ".log" or os.path.splitext(filename)[1] == ".dat"

def pathsToLDLogs() :
        resultPaths = []
        absRootPath = os.path.expanduser("~/.LD")
        if os.path.exists(absRootPath) :
                for account in os.listdir(absRootPath) :
                        absAccountPath = os.path.join(absRootPath, account)
                        if os.path.isdir(absAccountPath) :
                                for dirname in scanDirs:
                                        absLogsFolderPath = os.path.join(absAccountPath, dirname)
                                        for logFilename in os.listdir(absLogsFolderPath) :
                                                if includeFilename(logFilename) :
                                                        absLogPath = os.path.join(absLogsFolderPath, logFilename)
                                                        resultPaths.append((absRootPath, os.path.relpath(absLogPath, absRootPath))) # list of tuples (rootFolderPath, relFilePath)
        return resultPaths

def pathsToComponentLogs(maxNumFilesPerComponent = 5) :
        resultPaths = []
        absRootPath = os.path.expanduser("~/Library/Logs/Livedrive")
        if os.path.exists(absRootPath) :
                for component in os.listdir(absRootPath) :
                        absComponentPath = os.path.join(absRootPath, component)
                        if os.path.isdir(absComponentPath) :
                                logs = os.listdir(absComponentPath);
                                logs.sort(reverse=True); # newest first
                                numFilesFromComponent = 0
                                for logFilename in logs :
                                        if includeFilename(logFilename) :
                                                absLogPath = os.path.join(absComponentPath, logFilename)
                                                resultPaths.append((absRootPath, os.path.relpath(absLogPath, absRootPath))) # list of tuples (rootFolderPath, relFilePath)
                                                numFilesFromComponent = numFilesFromComponent + 1
                                                if numFilesFromComponent >=  maxNumFilesPerComponent:
                                                        print("Reached limit ({0}) on number of files from \"{1}\"".format(maxNumFilesPerComponent, absComponentPath));
                                                        break;
# only latest log
#                                if len(logs) :
#                                        logFilename = logs[-1] # take last item, latest date
#                                        if includeFilename(logFilename) :
#                                                absLogPath = os.path.join(absComponentPath, logFilename)
#                                                resultPaths.append((absRootPath, os.path.relpath(absLogPath, absRootPath))) # list of tuples (rootFolderPath, relFilePath)
        return resultPaths

def pathsToCrashLogs() :
        resultPaths = []
        absRootPath = os.path.expanduser("~/Library/Logs")
        if os.path.exists(absRootPath) :
                absReportsFolderPath = os.path.join(absRootPath, "DiagnosticReports")
                if os.path.exists(absReportsFolderPath) :
                        for logFilename in os.listdir(absReportsFolderPath) :
                                if includeFilename(logFilename) and "Livedrive" in logFilename :
                                        absLogPath = os.path.join(absReportsFolderPath, logFilename)
                                        resultPaths.append((absRootPath, os.path.relpath(absLogPath, absRootPath))) # list of tuples (rootFolderPath, relFilePath)
        return resultPaths

def pathsToSystemLogs() :
        resultPaths = []
        systemLogPath = os.path.expanduser("/private/var/log/system.log")
        if os.path.exists(systemLogPath) :
                resultPaths.append(("/private/var/log", "system.log"))
        return resultPaths

def pathsToPrefs() :
        resultPaths = []
        plistPath = os.path.expanduser("~/Library/Preferences/com.livedrive.livedriveapp.plist")
        if os.path.exists(plistPath) :
                resultPaths.append(("~/Library/Preferences", "com.livedrive.livedriveapp.plist"))
        return resultPaths

dstZipPath = os.path.expanduser("~/Desktop/Livedrive Logs.zip")

pathsToZip = []
pathsToZip.extend(pathsToPrefs())
pathsToZip.extend(pathsToCrashLogs())
pathsToZip.extend(pathsToComponentLogs(maxNumFilesPerComponent = 3))
pathsToZip.extend(pathsToLDLogs())
pathsToZip.extend(pathsToSystemLogs())

maxZipSize = 8 * 1024 * 1024
zipFiles(pathsToZip, dstZipPath, maxZipSize)
print("Archive is {0}".format(friendlySize(os.path.getsize(dstZipPath))))
